package mcwebhooks;

import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.entity.Entity;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.event.player.PlayerKickEvent;
import org.bukkit.event.player.PlayerLevelChangeEvent;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.event.entity.EntityDeathEvent;
import org.bukkit.event.entity.EntityDamageEvent.DamageCause;


import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.io.File;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

public final class MCWebHooks extends JavaPlugin implements Listener {
    private Config config;

    @Override
    public void onEnable() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            getLogger().info("Loading config");
            this.config = mapper.readValue(new File("plugins/webhooks.json"), Config.class);
            getLogger().info(String.format("Hooks will be sent to: %s", this.config.hookUrl));
        } catch (IOException e) {
            getLogger().info("Config loading failed");
            e.printStackTrace();
        }
        getServer().getPluginManager().registerEvents(this, this);
    }

    @Override
    public void onDisable() {
        getLogger().info("Web hooks are disabled");
    }

    @EventHandler
    public void onJoin(PlayerJoinEvent event) {
        JoinHookEvent hookEvent = new JoinHookEvent(event.getPlayer().getDisplayName());
        this.sendHookedEvent(hookEvent);
    }

    @EventHandler
    public void onQuit(PlayerQuitEvent event) {
        QuitHookEvent hookEvent = new QuitHookEvent(event.getPlayer().getDisplayName());
        this.sendHookedEvent(hookEvent);
    }

    @EventHandler
    public void onKick(PlayerKickEvent event) {
        QuitHookEvent hookEvent = new QuitHookEvent(event.getPlayer().getDisplayName());
        this.sendHookedEvent(hookEvent);
    }

    @EventHandler
    public void onEntityDeath(EntityDeathEvent event) {
        Entity ent = event.getEntity();
        EntityDamageEvent entDamageEvent = ent.getLastDamageCause();
        if (ent instanceof Player) {
            Player p = (Player)ent;
            DamageCause damageCause = entDamageEvent.getCause();
            DeathHookEvent hookEvent = new DeathHookEvent(p.getDisplayName(), damageCause.toString());
            this.sendHookedEvent(hookEvent);
        } else if (ent instanceof LivingEntity) {
            LivingEntity e = (LivingEntity)ent;
            if (e.getKiller() == null) {
                return;
            }
            PlayerKilledEntityEvent hookEvent = new PlayerKilledEntityEvent(
                e.getKiller().getDisplayName(),
                e.getName()
            );
            this.sendHookedEvent(hookEvent);
        }
    }

    @EventHandler
    public void onPlayerLevelChange(PlayerLevelChangeEvent event) {
        PlayerLevelChangeHookEvent hookEvent = new PlayerLevelChangeHookEvent(
            event.getPlayer().getDisplayName(),
            event.getNewLevel()
        );
        this.sendHookedEvent(hookEvent);
    }

    private final void sendHookedEvent(HookEvent event) {
        getLogger().info(String.format("Event: %s", event.type));
        ObjectMapper mapper = new ObjectMapper();
        HttpClient httpClient = HttpClientBuilder.create().build();
        HttpPost request = new HttpPost(this.config.hookUrl);
        request.addHeader("content-type", "application/json");
        try {
            StringEntity params = new StringEntity(mapper.writeValueAsString(event));
            request.setEntity(params);
        } catch (JsonProcessingException | UnsupportedEncodingException e1) {
            getLogger().info("Parsing failed");
            e1.printStackTrace();
            return;
        }
        try {
            httpClient.execute(request);
            getLogger().info("Request sent");
        } catch (IOException e) {
            getLogger().info("Request failed");
            e.printStackTrace();
        }
    }
}
